﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;


public class CallAttorneyAssignmentServiceCSharp
{
    const string USER_NAME = "YourUsername";
    const string PASSWORD = "YourPassword";

    static void Main(string[] args)
    {
        //This sample program can be used in two modes.   If "SUBMIT" is passed in as the command line argument
        //then it will format an update attorney assignment request and send it to the attorney assignment service.   Depending on the PushResponses argument
        //it will either specify an address to push response messages to or specify that responses are to be held for later retrieval.
        //If any other value is used for the command line argument it will attempt to pull any response messages that are available (assuming they were assed to be held).

        //Submit an attorney assignment message.
        if ((args.Length > 0) && (args[0] == "PUSHRESPONSES"))
        {
            //Submit the request specifying that you want responses pushed to you
            Submit(true);
            //Response will be pushed back to me at a later time
        }
        else
        {
            //Submit the request specifying that you will pull the responses
          Submit(false);
            //Pull all available responses.  
            PullResponses();
        }




    }

    //Submit the attorney assignment request
    static void Submit(Boolean ablnPushResponses)
    {

        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.ResponseTo objResponseToHeader = null;
        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.HoldResponse objHoldResponseHeader = null;
        //
        // Format an attorney assignment request and submit it to the attorney assignment service.
        //

        //
        //Create an instance of the service 
        //
        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.AttorneyAssignmentService_3_1Client objSvc = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.AttorneyAssignmentService_3_1Client("AttorneyAssignmentService_3_1_cu");

        //
        //Add a username token header to the request.
        //
        objSvc.ClientCredentials.UserName.UserName = USER_NAME;
        objSvc.ClientCredentials.UserName.Password = PASSWORD;

        //
        //depending on how responses are to be returned to the submittor set appropriate headers.
        //
        if (ablnPushResponses)
        {
            ////specify push responses to a web service
            objResponseToHeader = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.ResponseTo();
            objResponseToHeader.Address = "https://somewebservicereceiveraddress.asmx";
        }
        else
        {
            //Create a "HoldResponse" header and set its RetrievalCode value to "MyCompResponse".  This is used to specify that the
            //response messages should be "Held" and that they will be pulled at a later time.
            objHoldResponseHeader = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.HoldResponse();
            objHoldResponseHeader.RetrievalCode = "MyResponse";
        }

        //
        //Create the AttorneyAssignment request
        //
        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.UpdateAttorneyAssignment objUpdateAttorneyAssignment = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.UpdateAttorneyAssignment();
        objUpdateAttorneyAssignment.schemaVersion = "3:1";


        //Add the EFileID and Submitting Agency ORI
        objUpdateAttorneyAssignment.EFileID = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.EFileIDType();
        objUpdateAttorneyAssignment.EFileID.ID = "10100";
        objUpdateAttorneyAssignment.SubmittingAgencyORI = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.SubmittingAgencyORIType();
        objUpdateAttorneyAssignment.SubmittingAgencyORI.ID = "MN069033A";

        //add the court file number
        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.CourtFileNumberType objCFN = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.CourtFileNumberType();
        objUpdateAttorneyAssignment.Item = objCFN;
        objCFN.ID = "10-TR-03-3200";

        //specify that the assignment is for the defendant
        objUpdateAttorneyAssignment.PartyTypeText = Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.UpdateAttorneyAssignmentPartyTypeText.Defendant;

        //
        // Choose one of the following 2 sections to either assign or remove an attorney
        //

        //specify the assign attorney parameters
        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.UpdateAttorneyAssignmentAssignAttorney objAssignAttorney = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.UpdateAttorneyAssignmentAssignAttorney();
        objUpdateAttorneyAssignment.Item1 = objAssignAttorney;
        objAssignAttorney.AttorneyBarNumber = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.AttorneyBarNumberType();
        objAssignAttorney.AttorneyBarNumber.ID = "0131489";
        objAssignAttorney.AssignedDate = DateTime.Now.Date;
        objAssignAttorney.LeadAttorneyIndicator = true;
        objAssignAttorney.CourtAppointedIndicator = false;
        objAssignAttorney.PublicDefenderIndicator = true;

        ////specify the remove attorney parameters
        //Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.UpdateAttorneyAssignmentRemoveAttorney objRemoveAttorney = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.UpdateAttorneyAssignmentRemoveAttorney();
        //objUpdateAttorneyAssignment.Item1 = objRemoveAttorney;
        //objRemoveAttorney.AttorneyBarNumber = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.AttorneyBarNumberType();
        //objRemoveAttorney.AttorneyBarNumber.ID = "19X";
        //objRemoveAttorney.RemovedDate = DateTime.Now.Date;
        //objRemoveAttorney.RemovedReasonText = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.AttorneyRemovedReasonTypeCodeTextType();
        //objRemoveAttorney.RemovedReasonText.Value = "Substitution of Attorney"; //see the schema for the valid list of values for removed reason.
        

        //
        //If the address that is in the WSDL that the service was generated from is different from the
        //address you need to send the request to change its value in the app.config file.
        //

        //
        //Send the request
        //
        if (!objSvc.SubmitAttorneyAssignment(objHoldResponseHeader, objResponseToHeader, objUpdateAttorneyAssignment))
        {
            throw new System.Exception("Submission failed");
        }

    }

    //Pull any available attorney assignment response messages
    static void PullResponses()
    {

        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.PullReply objPullReply;
        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.Fault objSoap12Fault;
        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.Fault1 objSoap11Fault;
        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.UpdateAttorneyAssignmentResponse objAAResponse;
        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.UpdateAttorneyAssignmentResponseAcceptedByCourt objAcceptedByCourt;
        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.UpdateAttorneyAssignmentResponseRejected objRejected;
        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.CourtFileNumberType objCourtFileNumber;
        string strItemType;
        string strFaultCode;
        string strFaultMessage;

        //
        //Pull all available response messages with the retrieval code "MyResponse".   This only can be used if submissions used the HoldResponse header.
        //
        int intRemainingResponses = 1;
        int intPulledCount;

        //
        //Create an instance of the service 
        //
        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.AttorneyAssignmentService_3_1Client objSvc = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.AttorneyAssignmentService_3_1Client("AttorneyAssignmentService_3_1_cu");

        //
        //Add a username token header to the request.
        //
        objSvc.ClientCredentials.UserName.UserName = USER_NAME;
        objSvc.ClientCredentials.UserName.Password = PASSWORD;

        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.PullRequest objPullRequest = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.PullRequest();
        objPullRequest.schemaVersion = "3:1";
        objPullRequest.RetrievalCode = "MyResponse";
        //objPullRequest.RetrievalCode = "PUBDEF.STATE.MN.US";

        //
        //If the address that is in the WSDL that the service was generated from is different from the
        //address you need to send the request to change its value in the app.config file.
        //

        //
        //Declare variables to hold the output parameters efileid and submittingagencyori
        //
        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.SubmittingAgencyORIType objSubOri = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.SubmittingAgencyORIType();
        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.EFileIDType objEFileID = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.EFileIDType();


        //
        //Call the pull operation for each available response
        //
        while (intRemainingResponses > 0)
        {
            objEFileID = objSvc.PullHeldAttorneyAssignmentResponse(objPullRequest, out objSubOri, out objPullReply);

            intRemainingResponses = objPullReply.RemainingMessagesCount;
            intPulledCount = objPullReply.MessagePulledCount;
            if (intPulledCount >= 33)
            {
                //for some reason the same message has been pulled 3 or more times.   Someone should look at this (3 chosen as an example)
                throw new System.Exception("Message pulled 3 tiems");
            }


            //Check if a message was returned.   If it was then process it.
            if (objPullReply.Item != null)
            {
                //Process the response.   Item could be a Fault (11 or 12) or an AttorneyAssignmenttResponse
                strItemType = objPullReply.Item.GetType().ToString();

                if (strItemType == "Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.Fault")
                {
                    //got a soap11 fault
                    objSoap12Fault = (Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.Fault)objPullReply.Item;
                    strFaultCode = objSoap12Fault.Code.Subcode.Value.ToString();
                    strFaultMessage = objSoap12Fault.Reason[0].Value.ToString();
                    System.Diagnostics.Debug.WriteLine(strFaultCode + " " + strFaultMessage);
                    
                }
                else
                {
                    if (strItemType == "Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.Fault1")
                    {
                        //got a soap12 fault
                        objSoap11Fault = (Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.Fault1)objPullReply.Item;
                        strFaultCode = objSoap11Fault.faultcode.ToString();
                        strFaultMessage = objSoap11Fault.faultstring.ToString();
                        System.Diagnostics.Debug.WriteLine(strFaultCode + " " + strFaultMessage);
                    }
                    else
                    {
                        if (strItemType == "Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.UpdateAttorneyAssignmentResponse")
                        {
                            //got an attorney assignment response.   This indicates that the submission was successful
                            objAAResponse = (Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.UpdateAttorneyAssignmentResponse)objPullReply.Item;

                            if (objAAResponse.Item.GetType().ToString() == "Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.UpdateAttorneyAssignmentResponseRejected")
                            {
                                objRejected = (Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.UpdateAttorneyAssignmentResponseRejected)objAAResponse.Item;
                                System.Diagnostics.Debug.WriteLine("Assignment was rejeted: " + objRejected.ReasonText );
                            }
                            else
                            {
                                objAcceptedByCourt = (Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.UpdateAttorneyAssignmentResponseAcceptedByCourt)objAAResponse.Item;
                                objCourtFileNumber = (Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.CourtFileNumberType)objAcceptedByCourt.Item;
                                System.Diagnostics.Debug.WriteLine("Assignment Worked for case: " + objCourtFileNumber.ID);
                            }

                        }
                    }
                }


                //You can use the EFileID and SubmittingAgencyORI headers to link this pulled response with the original submission
                string strEFileID = objEFileID.ID;
                string strSubmittingAgencyORI = objSubOri.ID;

                //After processing the response release it so the next pull will get the next un-pulled message
                ReleaseResponse(objPullReply);

            }
        }

    }

    //Release an attorney assignment response message
    static void ReleaseResponse(Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.PullReply aobjPullReply)
    {

        //
        //Pull all available response messages with the retrieval code "MyResponse".   This only can be used if submissions used the HoldResponse header.
        //
        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.AttorneyAssignmentService_3_1Client objSvc = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.AttorneyAssignmentService_3_1Client("AttorneyAssignmentService_3_1_cu");

        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.ReleaseReply objReleaseReply;

        //
        //Add a username token header to the request.
        //
        objSvc.ClientCredentials.UserName.UserName = USER_NAME;
        objSvc.ClientCredentials.UserName.Password = PASSWORD;


        Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.ReleaseRequest objReleaseRequest = new Msc.Integration.ServiceCatalog.Sample.v4.AttorneyAssignmentService.ReleaseRequest();

        objReleaseRequest.schemaVersion = "3:1";
        objReleaseRequest.MessageID = aobjPullReply.MessageID;

        //
        //If the address that is in the WSDL that the service was generated from is different from the
        //address you need to send the request to change its value in the app.config file.
        //

        objReleaseReply = objSvc.ReleaseHeldAttorneyAssignmentResponse(objReleaseRequest);

    }

}

